import java.io.*;
/**
 * AbstractBoardGame implements common methods to
 * TicTacToe and Gomoku
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.4 1999-02-07
 */
public abstract class AbstractBoardGame implements BoardGame {

	protected char[][] gameState_;

	protected Player winner_ = new Player(); // nobody
	protected Player[] player_;
	protected int turn_ = X; // initial turn
	protected int squaresLeft_ = 9;

	static final int X = 0;
	static final int O = 1;

	/**
	 * set() and get() translate between chess coordinates
	 * and array indices.
	 */
	protected void set(char col, char row, char mark)
		throws AssertionException
	{
		assert(inRange(col, row));
		gameState_[col-'a'][row-'1'] = mark;
	}
	protected char get(char col, char row)
		throws AssertionException
	{
		assert(inRange(col, row));
		return gameState_[col-'a'][row-'1'];
	}

	/**
	 * Ask the current player to make a move.
	 */
	public void update() throws IOException {
		player_[turn_].move(this);
	}

	public Player currentPlayer() {
		return player_[turn_];
	}

	/**
	 * Called by the current player during an update().
	 * The player attempts to put a mark at a location.
	 */
	public void move(char col, char row, char mark)
		throws AssertionException
	{
		assert(this.notOver());
		assert(inRange(col, row));
		assert(this.get(col, row) == ' ');
		System.out.println(mark + " at " + col + row);
		this.set(col, row, mark);
		this.squaresLeft_--;
		this.swapTurn();
		this.checkWinner();
		assert(this.invariant());
	}

	protected void swapTurn() {
		turn_ = (turn_ == X) ? O : X;
	}
	
	public Player winner() {
		return winner_;
	}
	
	/**
	 * The game is not over as long as there is no winner
	 * and somebody can still make a move ...
	 */
	public boolean notOver() {
		return this.winner().isNobody()
			&& this.squaresLeft() > 0;
	}
	
	public int squaresLeft() {
		return this.squaresLeft_;
	}
	
	/**
	 * Check for a winning row, column or diagonal.
	 * (This code smells!  How can we clean it up?)
	 */
	protected void checkWinner()
		throws AssertionException
	{
		char player;
		for (char row='3'; row>='1'; row--) {
			player = this.get('a',row);
			if (player == this.get('b',row)
				&& player == this.get('c',row)) {
				this.setWinner(player);
				return;
			}
		}
		for (char col='a'; col <='c'; col++) {
			player = this.get(col,'1');
			if (player == this.get(col,'2')
				&& player == this.get(col,'3')) {
				this.setWinner(player);
				return;
			}
		}
		player = this.get('b','2');
		if (player == this.get('a','1')
			&& player == this.get('c','3')) {
				this.setWinner(player);
				return;
		}
		if (player == this.get('a','3')
			&& player == this.get('c','1')) {
				this.setWinner(player);
				return;
		}
	}
	
	/**
	 * Look up which player is the winner, and set winner_
	 * accordingly. Hm. Maybe we should store Players
	 * instead of chars in our array!
	 */
	protected void setWinner(char player) {
		if (player == ' ')
			return;
		if (player == player_[X].mark())
			winner_ = player_[X];
		else
			winner_ = player_[O];
	}

	/**
	 * A plain ascii representation of the game,
	 * mainly for debugging purposes.
	 */
	public String toString() {
		StringBuffer rep = new StringBuffer();
	
		for (char row='3'; row>='1'; row--) {
			rep.append(row);
			rep.append("  ");
			for (char col='a'; col <='c'; col++) {
				// NB: toString() is inherited from Object,
				// so we cannot override its signature by
				// throwing an AssertionException!
				try {
					rep.append(this.get(col,row));
				} catch (AssertionException err) {
					rep.append("Assertion violation: "
						+ err.getMessage());
				}
				if (col<'c') {
					rep.append(" | ");
				}
			}
			rep.append('\n');
			if (row>'1') {
				rep.append("  ---+---+---\n");
			}
		}
		rep.append("   a   b   c\n");
		return(rep.toString());
	}
	
	protected boolean inRange(char col, char row) {
		return (('a'<=col) && (col<='c')
			&& ('1'<=row) && (row<='3'));
	}

	protected void assert(boolean assertion)
		throws AssertionException
	{
		if (!assertion) {
				throw new AssertionException("assertion failed in "
					+ this.getClass().getName());
		}
	}
	
	/**
	 * These seem obvious, which is exactly why
	 * they should be checked.
	 */
	private boolean invariant() {
		return (turn_ == X || turn_ == O)
			&& (this.notOver() 
				|| this.winner() == player_[X]
				|| this.winner() == player_[O]
				|| this.winner().isNobody())
			&& (squaresLeft_ < 9
				// else, initially:
				|| turn_ == X && this.winner().isNobody());
	}

	/**
	 * For now, just test some obvious things.
	 * Do we need to test toString()?  How?
	 */
	public void test() {
		try {
			System.err.println("Started TicTacToe tests");
			assert(this.invariant());
			assert(this.get('a','1') == ' ');
			assert(this.get('c','3') == ' ');
			this.set('c','3','X');
			assert(this.get('c','3') == 'X');
			this.set('c','3',' ');
			assert(this.get('c','3') == ' ');
			assert(!this.inRange('d','4'));
			assert(this.invariant());
			System.err.println("Passed TicTacToe tests");
		} catch (AssertionException err) {
			System.err.println("TicTacToe assertion failed!");
			err.printStackTrace();
			// System.exit(1);
		}
	}
}
