package tictactoe;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.util.*;

/**
 * This class implements the view on a TicTacToe game.
 * The Applet contains a grid of Place objects that
 * intercept mouse clicks and forward them to the game's
 * Players.  The Applet is also an Observer for the game.
 * When the game notifies the Applet of state change, it
 * updates the corresponding Place.  Below the grid there
 * is a Label component to display messages.
 * <P>
 * <B>To do:</B>
 * <UL>
 * <LI>add a mechanism to select TicTacToe or Gomoku
 * <LI>visualize automated tests?
 * </UL>
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.0 1999-03-26
 */
public class GameApplet extends Applet implements Observer
{
	private BoardGame game_;
	private Label label_;
	private Place[][] places_;
	
	static private final int MINSIZE = 50;
	
	/**
	 * Return author and copyright information
	 */
	public String getAppletInfo() {
		return "Author Oscar.Nierstrasz@acm.org (co) 1999";
	}

	/**
	 * Instantiates the BoardGame, initializes the view components,
	 * and connects the view to the model.
	 */
	public void init() {
		game_ = makeGame();				// instantiate game

		setLayout(new BorderLayout());	// initialize view
		setSize(MINSIZE*game_.cols(),MINSIZE*game_.rows());

		add("North", makeControls());

		add("Center", makeGrid());
		
		label_ = new Label();
		add("South", label_);

		game_.addObserver(this);		// connect to model

		showFeedBack(game_.currentPlayer().mark() + " plays");
	}

	/**
	 * @return the BoardGame associated with this Applet
	 */
	public BoardGame game() { return game_; }
	
	/**
	 * Creates BoardGame.  Missing a mechanism to instantiate
	 * Gomoku instead of TicTacToe.
	 * @see AppletPlayer
	 */
	private BoardGame makeGame() {
		Player X = new AppletPlayer('X');
		Player O = new AppletPlayer('O');
		return new TicTacToe(X, O);
		// return new Gomoku(X, O);
		// return new Gomoku(X, O, 10, 5);
	}

	private Component makeControls() {
		Button again = new Button("New game");
		again.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// methods of enclosing class (GameApplet)
				showFeedBack("starting new game ...");
				newGame();
			}
		});
		return again;
	}

	/**
	 * Make the grid that serves as a view and controller for the game.
	 */
	private Component makeGrid() {
		int cols = game_.cols();
		int rows = game_.rows();
		Image xImage = getImage(getCodeBase(), "images/cross.gif");
		Image oImage = getImage(getCodeBase(), "images/not.gif");

		Panel grid = new Panel();
		
		grid.setLayout(new GridLayout(rows, cols));

		places_ = new Place[cols][rows];
		for (int row=rows-1; row>=0; row--) {
			for (int col=0; col<cols; col++) {
				Place p = new Place(col, row, xImage, oImage);
				p.addMouseListener(new PlaceListener(p, this));
				p.setBackground(Color.white);
				grid.add(p);
				places_[col][row] = p;
			}
		}
		return grid;
	}
	
	/**
	 * Start a new game by deleting the old one, instantiating
	 * a new one, and clearing the images of the Places.
	 */
	private void newGame() {
		game_ = makeGame();
		game_.addObserver(this);

		int cols = game_.cols();
		int rows = game_.rows();

		for (int col=cols-1; col>=0; col--) {
			for (int row=0; row<rows; row++) {
				places_[col][row].clearImage();
			}
		}
		showFeedBack(game_.currentPlayer().mark() + " plays");
	}

	/**
	 * Displays a feedback string in the associated Label component.
	 * @param msg the string to display
	 */
	public void showFeedBack(String msg) {
		label_.setText(msg);
	}

	/**
	 * Implements the java.util.Observer interface.
	 * Called by the BoardGame when its state changes.
	 * @param o an instance of a BoardGame
	 * @param arg an instance of Move
	 @ @see Move
	 */
	public void update(Observable o, Object arg) {
		Move move = (Move) arg;
		showFeedBack("got an update: " + move);
		places_[move.col][move.row].setMove(move.player);
	}
}
