package server;

import java.io.*;
import java.util.*;
import java.rmi.*;

/**
 * Remote interface for TicTacToe and Gomoku.
 *
 * This interface adapts the BoardGame interface.
 * All RemoteGame methods throw RemoteException,
 * and none of them throws an AssertionException.
 * BoardGame methods that return a Player are
 * adapted here so they return a char representing
 * the Player (since a Player is not a remote object).
 *
 * This interface also allows a client to register
 * as a RemoteObserver of the game.
 *
 * @see tictactoe.GameProxy
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
public interface RemoteGame extends Remote {
	/**
	 * The game is ready to be played.
	 * (There are two connected clients.)
	 **/
	public boolean ready()
		throws RemoteException;
	/**
	 * A client connected to the game must join
	 * the game to know what its mark is (X or O).
	 **/
	public char join()
		throws RemoteException;
	/**
	 * Attempt to move to the given square.
	 * Called by a remote client.  The RemoteGame
	 * will check if move.mark corresponds to the
	 * current Player, and then call BoardGame.move().
	 *
	 * @return true if move is ok
	 **/
	public boolean move(Move move)
		throws RemoteException;

	/**
	 * @return the number of columns of this game
	 **/
	public int cols()
		throws RemoteException;
	/**
	 * @return the number of rows
	 **/
	public int rows()
		throws RemoteException;
	/**
	 * @return the mark of the Player whose turn it is.
	 **/
	public char currentPlayer()
		throws RemoteException;
	/**
	 * @return the name of winner of the game (possibly "nobody")
	 **/
	public String winner()
		throws RemoteException;
	/**
	 * @return whether the game is over
	 **/
	public boolean notOver()
		throws RemoteException;
	/**
	 * Register as a RemoteObserver of the game.
	 * @see server.RemoteObserver
	 * @see client.GameObserver
	 **/
	public void addObserver(RemoteObserver o)
		throws RemoteException;
}
