package tictactoe;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;

import server.*;

/**
 * BoardGame wrapper to provide RemoteGame interface.
 *<P>
 * <I>NB:</I> All methods are synchronized since multiple
 * client threads may access the server simultaneously.
 *
 * @see tictactoe.AbstractBoardGame
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
public class GameProxy 
		extends UnicastRemoteObject
		implements RemoteGame
{
	// the game we wrap
	protected BoardGame game_;
	// number of clients connected
	protected int connected_ = 0;

	/**
	 * Must be declared since we inherit from UnicastRemoteObject
	 **/
	GameProxy() throws RemoteException { super(); }

	/**
	 * Use this constructor.  A GameProxy is just
	 * a wrapper around an existing BoardGame.
	 * Several methods are just boilerplate delegations
	 * to game_.
	 **/
	GameProxy(BoardGame game)
		throws RemoteException
	{
		game_ = game;
	}

	/**
	 * The game is ready to be played if
	 * there are two connected clients.
	 **/
	public synchronized boolean ready()
		throws RemoteException
	{
		return connected_ >= 2;
	}

	/**
	 * A client connected to the game must join
	 * the game to know what its mark is (X or O).
	 * If two players are connected, further players
	 * may connect, but only as spectators.
	 * (This possibility is not exploited in 3.0)
	 **/
	public synchronized char join()
		throws RemoteException
	{
		// look up mark of this player, then increment
		return game_.player(connected_++).mark();
	}

	/**
	 * Attempt to move to the given square.
	 * Called by a remote client.
	 * We check if move.mark corresponds to the
	 * current Player, and then call BoardGame.move().
	 *
	 * @see client.PlaceListener#mouseClicked
	 *
	 * @return true if the move is valid
	 **/
	public synchronized boolean move(Move move)
		throws RemoteException
	{
		Player current = game_.currentPlayer();
		if (current.mark() != move.mark)
			return false;
		try {
			game_.move(move.col, move.row, current);
			return true;
		} catch (AssertionException e) {
			return false;
		}
	}

	/**
	 * For remote clients.
	 * @return mark of current player (i.e., not the Player)
	 **/
	public synchronized char currentPlayer()
		throws RemoteException
	{
		return game_.currentPlayer().mark();
	}

	/**
	 * For remote clients.
	 * @return the name of the winner (i.e., not the Player)
	 **/
	public synchronized String winner()
		throws RemoteException
	{
		return game_.winner().toString();
	}

	// The rest are boilerplate delegation methods

	public synchronized void addObserver(RemoteObserver remote)
		throws RemoteException
	{ game_.addObserver(remote); }

	public synchronized int cols() throws RemoteException
	{ return game_.cols(); }

	public synchronized int rows() throws RemoteException
	{ return game_.rows(); }

	public synchronized boolean notOver() throws RemoteException
	{ return game_.notOver(); }

}
